# This program can be used to reproduce the simulation results appearing in Appendix E.4 (supplement 
# p. 46) of Escanciano & Goh (2018), 
# ``Quantile-Regression Inference with Adaptive Control of Size''

# Users will at minimum need to specify:
#   the sample size ('n'; the paper used n=100 and n=300)
#   the quantile of interest ('qtile'; the paper used qtile=.25, .50, .75)
#   the error distribution ('err.dist'; the paper used err.dist= 1)
#   the specification of the parameter delta; corresponding to the DGPs described in the text as 
#     Model 1, 2, 3, 4, 5 or 6 ('alt'; the paper used alt=1, 2, 3, 4, 5, 6)

# Basic parameters
n<- 100           # sample size
qtile <- 0.5      # quantile of interest
err.dist<- 1               # error distribution: 1=normal, 2=t(df1), 3=chi2(df2)
alt<- 1                    # alternatives: 1=loc, 2=loc-scale, 3--5=beta, 6= sin
km <- 5                  # leading constant used in specifying the number $m$ of random quantiles used in our procedure
                         # we have found that km=5 works well in practice
m <- floor(km*(n/(log(n)^(11/5)))^(5/4))  # number of random quantiles used in our procedure
tau0<-0                   # lower bound for alternatives, 0.1 less than qtile
tau1<-1                   # upper bound for alternatives, 0.1 more than qtile
ncov<- 5                  # number of covariates
np <- ncov+3           # number of variables+intercept
cgrid <- c(0,.50,1.0,1.50) # cgrid indexes alternatives
num.reps <- 1000           # number of Monte Carlo replications
boot.reps <- 200           # number of bootstrap replications for bxy, bpwy, bmcmb, bwxy, bwild
df1<-3                    # degrees of freedom for t
df2<-1                    # degrees of freedom for chi2
bw.const <- 1.5           # proportionality constant for fixed bandwidth
                          # we have found that bw.const=1.5 works well in practice
df.test <- 2              # number of hypothesized linear restrictions

rm(.Random.seed, envir=globalenv()) # reset random seed

# R libraries that need to be installed on your system
library(quantreg)
library(mvtnorm)
library(ks)
library(MASS)
library(purrr)
library(xtable)

# Endpoints of interval $[a1,a2]$ over which the random quantiles used in our procedure are drawn as 
# iid uniforms, i.e., our procedure uses $m$ iid random variates from a $Unif[a1,a2]$-distribution
a1 <- 0
a2 <- 1

# The eg.proc.2 function implements the proposed estimate of 
# the G matrix for regression quantile standard errors with a fixed bandwidth

eg.proc.2 <- function(R,X,y,qtile,a1,a2,km=1, bw.const=1, qtiles.provided=FALSE, qtiles) { 
  # X should not contain an intercept
  betahat <- rq(y~X, tau=qtile)
  info <- t(betahat$coefficients)
  n <- length(y)
  Xplus <- cbind(rep(1,n),X)
  d <- dim(Xplus)[2]
  qn <- n^(-1)*crossprod(Xplus,Xplus)
  m <- floor(km*(n/(log(n)^(11/5)))^(5/4))
  
  if (qtiles.provided==FALSE){
    unifs <- runif(m, min=a1,max=a2)
    unifs <- sort(unifs)
  } else {
    unifs <- qtiles
  }
  betahat.grid <- rq(y~X, tau=unifs)
  info.grid <-t(betahat.grid$coefficients)
  
  heg <- abs(bw.const*(log(m)/m)^(1/5))
  
  kern <- .75*(matrix(rep(1,n*m),nrow=n)-(heg^(-1)*(Xplus %*% t(info.grid)-Xplus %*% kronecker(t(info),matrix(rep(1,m),nrow=1))))^2)*(abs(heg^(-1)*(Xplus %*% t(info.grid)-Xplus %*% kronecker(t(info),matrix(rep(1,m),nrow=1))))<=1)
  
  blah <- (a2-a1)*heg^(-1)*kern
  fhateg <- diag(rowMeans(blah))
  ghateg <- n^(-1)*t(Xplus) %*% fhateg %*% Xplus
  return(ghateg)
}

# the eg.proc procedure returns the \hat{G} matrix along with the suggested data-driven bandwidth 
# appearing in our paper
eg.proc <- function(R,X,y,qtile,a1,a2,m.fixed=TRUE,m,km=1, qtiles.provided=FALSE, qtiles) { 
  # X should not contain an intercept
  betahat <- rq(y~X, tau=qtile)
  info <- t(betahat$coefficients)
  n <- length(y)
  Xplus <- cbind(rep(1,n),X)
  d <- dim(Xplus)[2]
  qn <- n^(-1)*crossprod(Xplus,Xplus)
  if (m.fixed==FALSE) {
    m <- floor(km*(n/(log(n)^(11/5)))^(5/4))
  }

  if (qtiles.provided==FALSE){
    unifs <- runif(m, min=a1,max=a2)
    unifs <- sort(unifs)
  } else {
    unifs <- qtiles
  }

  betahat.grid <- rq(y~X, tau=unifs)
  info.grid <-t(betahat.grid$coefficients)


  prelim <- lm(y~X)
  info.resid <- prelim$residuals
  info.resid <- sort(info.resid, decreasing=FALSE, index.return=TRUE)
  bottom <- floor(n*a1)
  top <- floor(n*a2)
  if (top-bottom <=d) {
    bottom <- floor(n*(qtile-d/(2*n)))
    top <- floor(n*(qtile+d/(2*n)))
  }
  y.adj <- y[c(info.resid$ix[c(-(1:bottom),-(top+1:n))])]
  X.adj <- X[c(info.resid$ix[c(-(1:bottom),-(top+1:n))]),]
  n.adj <- length(y.adj)
      
  bmid <- rq(y.adj~X.adj, tau=qtile)$coefficients
  X.adj <- cbind(rep(1,n.adj),X.adj)
  xbhat <- X.adj %*% bmid
      
  xbhat.mat <- X.adj %*% t(info.grid)
      
  fhat.vec <- numeric(n.adj)
  f2hat.vec <- numeric(n.adj)
  hpre <- m^(-1/5)
  hpre2 <- m^(-1/9)
  for (ii in 1:n.adj) { 
    fhat.vec[ii] <- kdde(x=xbhat.mat[ii,], eval.points=xbhat[ii],deriv.order=0, h=hpre)$estimate
    f2hat.vec[ii] <- kdde(x=xbhat.mat[ii,], eval.points=xbhat[ii],deriv.order=2,h=hpre2)$estimate
  }
      
  fhat <- diag(fhat.vec)
  f2hat <- diag(f2hat.vec)
  t1hat <- (n.adj^(-1))*t(X.adj) %*% f2hat %*% X.adj
  sqrtfhat <- diag(sqrt(fhat.vec))
  t2hat <- (n.adj^(-1))*t(X.adj) %*% sqrtfhat %*% X.adj
  ghat <- (n.adj^(-1))*t(X.adj) %*% fhat %*% X.adj
  u1hat <- ginv(R %*% ginv(ghat) %*% qn %*% ginv(ghat) %*% t(R)) %*% (R %*% (ginv(ghat) %*% t1hat %*% ginv(ghat) %*% qn %*% ginv(ghat)+ginv(ghat) %*% qn %*% ginv(ghat) %*% t1hat %*% ginv(ghat))%*% t(R))%*% ginv(R %*% ginv(ghat) %*% qn %*% ginv(ghat) %*% t(R))
  lambda1hat <- info %*% t(R) %*% u1hat %*% R %*% t(info)
  u2hat <- ginv(R %*% ginv(ghat) %*% qn %*% ginv(ghat) %*% t(R)) %*% (R %*% (ginv(ghat) %*% t2hat %*% ginv(ghat) %*% qn %*% ginv(ghat)+ginv(ghat) %*% qn %*% ginv(ghat) %*% t2hat %*% ginv(ghat)) %*% t(R)) %*% ginv(R %*% ginv(ghat) %*% qn %*% ginv(ghat) %*% t(R))
  lambda2hat <- info %*% t(R) %*% u2hat %*% R %*% t(info)
  
  const.bw <- (lambda2hat^2/(lambda1hat^2))^(1/5)
  heg <- const.bw*(log(m)/m)^(1/5) # rate-optimal bandwidth
  heg <- c(heg)
  kern <- .75*(matrix(rep(1,n*m),nrow=n)-(heg^(-1)*(Xplus %*% t(info.grid)-Xplus %*% kronecker(t(info),matrix(rep(1,m),nrow=1))))^2)*(abs(heg^(-1)*(Xplus %*% t(info.grid)-Xplus %*% kronecker(t(info),matrix(rep(1,m),nrow=1))))<=1)
  
  blah <- (a2-a1)*heg^(-1)*kern
  fhateg <- diag(rowMeans(blah))
  ghateg <- n^(-1)*t(Xplus) %*% fhateg %*% Xplus
  eg.proc.results <- list(const.bw,heg,ghateg)
  return(eg.proc.results)
}

# Initialize vectors of test statistics
fstat1.weg <- numeric(num.reps)
fstat1.weg.fixed <- numeric(num.reps)
fstat1.wiid <- numeric(num.reps)
fstat1.wnid <- numeric(num.reps)
fstat1.wker <- numeric(num.reps)
fstat1.bxy <- numeric(num.reps)
fstat1.bpwy <- numeric(num.reps)
fstat1.bmcmb <- numeric(num.reps)
fstat1.bwxy <- numeric(num.reps)
fstat1.bwild <- numeric(num.reps)
fstat1.wiid.anova <- numeric(num.reps)
fstat1.wnid.anova <- numeric(num.reps)
fstat1.wker.anova <- numeric(num.reps)
fstat1.riid.anova <- numeric(num.reps)




# Initialize total number of rejections across Monte Carlo replications
rej1.weg <- 0
rej1.weg.fixed <- 0
rej1.wiid <- 0
rej1.wnid <- 0
rej1.wker <- 0
rej1.bxy <- 0
rej1.bpwy <- 0
rej1.bmcmb <- 0
rej1.bwxy <- 0
rej1.bwild <- 0
rej1.wiid.anova <- 0
rej1.wnid.anova <- 0
rej1.wker.anova <- 0
rej1.riid.anova <- 0


# Quantities to carry over between Loops 1 and 2
u.mat <- matrix(0,nrow=n,ncol=num.reps)
D.mat <- matrix(0,nrow=n,ncol=num.reps)
x1.mat <- matrix(0,nrow=n,ncol=num.reps)
x2.mat <- matrix(0,nrow=n,ncol=num.reps)
x3.mat <- matrix(0,nrow=n,ncol=num.reps)
x4.mat <- matrix(0,nrow=n,ncol=num.reps)
x5.mat <- matrix(0,nrow=n,ncol=num.reps)
weg.unif.mat <- matrix(0,nrow=m,ncol=num.reps)

# Loop 1: computation of Wald-type test statistics and their rejections for significance of treatment indicator
for (r in 1:num.reps) {
  cat(paste(" Replication", r, "of", num.reps, "...\n"))
  rand.all <- mvrnorm(n,mu=rep(0,ncov+2),Sigma=diag(rep(1,ncov+2)))
  u <- pnorm(rand.all[,ncov+1])
  u.mat[,r] <- u
# Generate quantiles for the error distribution 
  if (err.dist==1) { 
  qu <- qnorm(u)
  }
  if (err.dist==2) { 
  qu <- qt(u,df1)  
  }
  if (err.dist==3) { 
  qu <- qchisq(u, df2)-df2
  }
 # Generate different deltas 
  if (alt<=5) { 
    delta <- 0 
  }
  if (alt==6) {
    delta <- (sin(2*pi*u)-sin(2*pi*qtile))/(2*pi) # sin
  }

  DD <- pnorm(rand.all[,ncov+2])
  D <-  qbinom(DD, size=1, prob=0.5)
  D.mat[,r] <- D
  if (ncov>1) {
    X.all <- rand.all[,1:ncov]
    x1.mat[,r] <- X.all[,1]
    x2.mat[,r] <- X.all[,2]
    x3.mat[,r] <- X.all[,3]
    x4.mat[,r] <- X.all[,4]
    x5.mat[,r] <- X.all[,5]
    
    xsum=apply(X.all[,1:4],1,sum)
    y <- 1+xsum+D+D*X.all[,1]*delta+qu
    xv <- cbind(X.all[,1:4],D,D*X.all[,1],X.all[,5])
    xv0 <- cbind(X.all[,1:4],D)
  }

  if (ncov==0) {
    y <- 1+D*delta+qu
    xv <- cbind(D)
  }
     
  xmat <- cbind(c(rep(1,n)),xv)

  betahat <- rq(y~xv, tau=qtile)
  betahat.R0 <- rq(y~X.all[,1]+X.all[,2]+X.all[,3]+X.all[,4]+D+D*X.all[,1]+X.all[,5], tau=qtile)  # unrestricted model
  betahat.R1 <- rq(y~X.all[,1]+X.all[,2]+X.all[,3]+X.all[,4]+D, tau=qtile)  # restricted (i.e., null model)

  R1 <- rbind(c(rep(0,ncov+1),1,0), c(rep(0,ncov+1),0,1))
  qtiles <- runif(m, min=a1,max=a2)
  qtiles <- sort(qtiles)
  weg.unif.mat[,r] <- qtiles
  info <- summary.rq(betahat, se="iid", hs=TRUE)$coefficients # Model 1 Wiid
  
  # Model 1 Weg (i.e., proposed procedure)
    
  hn.list <- eg.proc(R1,xv,y,qtile,a1,a2, m.fixed=FALSE,m,km,qtiles.provided=TRUE,qtiles) # data-driven bandwidth
  weg.bw.const.grid[r] <- unlist(hn.list[1])
  hn <-matrix(unlist(hn.list[3]),ncol=np,byrow=TRUE)
  jn <- n^(-1)*crossprod(xmat,xmat)
  
  possibleError <- tryCatch({  vhat <- qtile*(1-qtile)*ginv(hn) %*% jn %*% ginv(hn)
  }, error=function(e) e)
  if (!inherits(possibleError, "error")){
    fstat1.weg[r] <- n*t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
    rej1.weg <- rej1.weg +(n*t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))
  }
  
  
  
  hn.fixed <- eg.proc.2(R1,xv,y,qtile,a1,a2,km, bw.const,qtiles.provided=TRUE,qtiles) 
  vhat <- qtile*(1-qtile)*ginv(hn.fixed) %*% jn %*% ginv(hn.fixed)
  
  fstat1.weg.fixed[r] <- n*t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.weg.fixed <- rej1.weg.fixed +(n*t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))


  info <- summary.rq(betahat, se="iid", hs=TRUE)$coefficients # Model 1 Wiid
  vhat <- summary.rq(betahat, se="iid", hs=TRUE, covariance=TRUE)$cov
  
  fstat1.wiid[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.wiid <- rej1.wiid +(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))
    

  anova.wiid <- anova(betahat.R0,betahat.R1,test="Wald", se="iid")
  fstat1.wiid.anova[r] <- anova.wiid[[1]][3]
  rej1.wiid.anova <- rej1.wiid.anova+(anova.wiid[[1]][4]<.05)

  
  
  possibleError <- tryCatch({info <- summary.rq(betahat, se="nid", hs=TRUE)$coefficients # Model 1 Wnid
  }, error=function(e) e)
  if (!inherits(possibleError, "error")){
    fstat1.wnid[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
    rej1.wnid <- rej1.wnid+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))
    

    anova.wnid <- anova(betahat.R0,betahat.R1,test="Wald", se="nid")
    fstat1.wnid.anova[r] <- anova.wnid[[1]][3]
    rej1.wnid.anova <- rej1.wnid.anova+(anova.wnid[[1]][4]<.05)
  }

  
  info <- summary.rq(betahat, se="ker", hs=TRUE)$coefficients # Model 1 Wker
  vhat <- summary.rq(betahat, se="ker", hs=TRUE, covariance=TRUE)$cov
  
  fstat1.wker[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.wker <- rej1.wker+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))


  anova.wker <- anova(betahat.R0,betahat.R1,test="Wald", se="ker")
  fstat1.wker.anova[r] <- anova.wker[[1]][3]
  rej1.wker.anova <- rej1.wker.anova+(anova.wker[[1]][4]<.05)

  info <- summary.rq(betahat, se="rank", iid=TRUE, alpha=.05) # Model 1 Riid
  info <- info$coefficients
  
  anova.riid <- anova(betahat.R0,betahat.R1,test="rank")
  fstat1.riid.anova[r] <- anova.riid[[1]][3]
  rej1.riid.anova <- rej1.riid.anova+(anova.riid[[1]][4]<.05)

  info.boot <- summary.rq(betahat, se="boot", bsmethod="xy",R=boot.reps, covariance=TRUE) # Model 1 Bxy
  info <- info.boot$coefficients 
  vhat <- info.boot$cov
  
  fstat1.bxy[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.bxy <- rej1.bxy+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))


  info.boot <- summary.rq(betahat, se="boot", bsmethod="pwy", R=boot.reps, covariance=TRUE) # Model 1 Bpwy
  info <- info.boot$coefficients 
  vhat <- info.boot$cov
  
  fstat1.bpwy[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.bpwy <- rej1.bpwy+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))


  possibleError <- tryCatch({info.boot <- summary.rq(betahat, se="boot", bsmethod="mcmb", R=boot.reps, covariance=TRUE) # Model 1 Bmcmb
  }, error=function(e) e)
  if (!inherits(possibleError, "error")){
    info <- info.boot$coefficients 
    vhat <- info.boot$cov
    fstat1.bmcmb[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
    rej1.bmcmb <- rej1.bmcmb+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95, df=df.test))
  }

  info.boot <- summary.rq(betahat, se="boot", bsmethod="wxy", R=boot.reps, covariance=TRUE) # Model 1 Bwxy
  info <- info.boot$coefficients 
  vhat <- info.boot$cov
  
  fstat1.bwxy[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.bwxy <- rej1.bwxy+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))


  info.boot <- summary.rq(betahat, se="boot", bsmethod="wild", R=boot.reps, covariance=TRUE) # Model 1 Bwild
  info <- info.boot$coefficients 
  vhat <- info.boot$cov
  
  fstat1.bwild[r] <- t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])
  rej1.bwild <- rej1.bwild+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>qchisq(p=.95,df=df.test))
  

} #end Loop 1

# Compute empirical sizes
size1weg <- rej1.weg/num.reps
size1weg.fixed <- rej1.weg.fixed/num.reps
size1wiid <- rej1.wiid/num.reps
size1wnid <- rej1.wnid/num.reps
size1wker <- rej1.wker/num.reps
size1wiid.anova <- rej1.wiid.anova/num.reps
size1wnid.anova <- rej1.wnid.anova/num.reps
size1wker.anova <- rej1.wker.anova/num.reps
size1riid.anova <- rej1.riid.anova/num.reps
size1bxy <- rej1.bxy/num.reps
size1bpwy <- rej1.bpwy/num.reps
size1bmcmb <- rej1.bmcmb/num.reps
size1bwxy <- rej1.bwxy/num.reps
size1bwild <- rej1.bwild/num.reps

size1.raw <- c(size1weg, size1weg.fixed, size1wiid, size1wiid.anova, size1wnid, size1wnid.anova, size1wker, size1wker.anova, size1riid.anova, size1bxy, size1bpwy, size1bmcmb, size1bwxy, size1bwild)
size1.raw <- round(size1.raw, digits=4)


rowlabel <- c("weg", "weg (fixed)", "wiid", "wiid (anova)", "wnid", "wnid (anova)", "wker", "wker (anova)", "riid (anova)", "bxy", "bpwy", "bmcmb", "bwxy", "bwild")

table.size <- cbind(rowlabel, size1.raw)

# LaTeX ready table:  Empirical sizes for nominal 5\%-tests based on N(0,1)-critical values.  
xtable(table.size, caption="QTE, n=100, 1000 replications, quantile=.5", digits=c(4))
save(table.size, file=paste("qdf_qte_int_F2tab1_n",toString(n),"_q",toString(qtile),"_e",toString(err.dist),"_a",toString(alt),".txt",sep=""))




# Empirical tail quantiles of each Wald-type test statistic, used to compute empirical size-adjusted
# powers for level-.05 tests
cv951.weg <- quantile(fstat1.weg,probs=.95)
cv951.weg.fixed <- quantile(fstat1.weg.fixed,probs=.95)
cv951.wiid <- quantile(fstat1.wiid,probs=.95)
cv951.wnid <- quantile(fstat1.wnid,probs=.95)
cv951.wker <- quantile(fstat1.wker,probs=.95)
cv951.bxy <- quantile(fstat1.bxy,probs=.95)
cv951.bpwy <- quantile(fstat1.bpwy,probs=.95)
cv951.bmcmb <- quantile(fstat1.bmcmb,probs=.95)
cv951.bwxy <- quantile(fstat1.bwxy,probs=.95)
cv951.bwild <- quantile(fstat1.bwild,probs=.95)

fstat1.wiid.anova <- as.numeric(fstat1.wiid.anova)
fstat1.wnid.anova <- as.numeric(fstat1.wnid.anova)
fstat1.wker.anova <- as.numeric(fstat1.wker.anova)
fstat1.riid.anova <- as.numeric(fstat1.riid.anova)
fstat1.wiid.anova <- df.test*fstat1.wiid.anova
fstat1.wnid.anova <- df.test*fstat1.wnid.anova
fstat1.wker.anova <- df.test*fstat1.wker.anova
fstat1.riid.anova <- df.test*fstat1.riid.anova

cv951.wiid.anova <- quantile(fstat1.wiid.anova,probs=.95)
cv951.wnid.anova <- quantile(fstat1.wnid.anova,probs=.95)
cv951.wker.anova <- quantile(fstat1.wker.anova,probs=.95)
cv951.riid.anova <- quantile(fstat1.riid.anova,probs=.95)

# Initialize number of rejections under the alternatives considered
rej2.riid.anova <- numeric(length(cgrid)-1)
rej2.weg <- numeric(length(cgrid)-1)
rej2.weg.fixed <- numeric(length(cgrid)-1)
rej2.wiid <- numeric(length(cgrid)-1)
rej2.wnid <- numeric(length(cgrid)-1)
rej2.wker <- numeric(length(cgrid)-1)
rej2.wiid.anova <- numeric(length(cgrid)-1)
rej2.wnid.anova <- numeric(length(cgrid)-1)
rej2.wker.anova <- numeric(length(cgrid)-1)
rej2.bxy <- numeric(length(cgrid)-1)
rej2.bpwy <- numeric(length(cgrid)-1)
rej2.bmcmb <- numeric(length(cgrid)-1)
rej2.bwxy <- numeric(length(cgrid)-1)
rej2.bwild <- numeric(length(cgrid)-1)


# Loop 2:  Computation of empirical size-adjusted power for nominal 5\% tests of signficance of the 
# treatment indicator.  Power is against each alternative corresponding to the non-zero values in the 
# vector cgrid
for (r in 1:num.reps) {
  cat(paste(" Replication", r, "of", num.reps, "...\n"))
  for (cc in 2:length(cgrid)){
    u <- u.mat[,r]

    # Generate quantiles for the error distribution 
    if (err.dist==1) { 
      qu <- qnorm(u)
    }
    else if (err.dist==2) { 
      qu <- qt(u,df1)
    }
    else { 
      qu <- qchisq(u, df2)-df2
    }

    # Generate different alternatives 
    if (alt==1) { 
      delta <- cgrid[cc]*(tau0<=u& u<=tau1) # Location
    }
    else if (alt==2) { 
      delta <- cgrid[cc]*(1+qu)*(tau0<=u& u<=tau1) # Location-Scale
    }
    else if (alt==3) { 
      s1<-1
      s2<-4
      qq<-5*qbeta(u,s1,s2)
      delta <- (qbeta(u,s1,s2)-qbeta(qtile,s1,s2)-cgrid[cc]*qq)*(tau0<=u& u<=tau1) # 
    }
    else if (alt==4) { 
      s1<-0.5
      s2<-0.5
      delta <- cgrid[cc]*2*qbeta(u,s1,s2)*(tau0<=u& u<=tau1) # 
    }
    else if (alt==5) { 
      s1<-2
      s2<-2
      delta <- cgrid[cc]*2*qbeta(u,s1,s2)*(tau0<=u& u<=tau1) # 
    }
    else {
      delta <- (sin(2*pi*u)-sin(2*pi*qtile)-cgrid[cc]*2*pi)/(2*pi) # sin
    }
    D <-  D.mat[,r]
    if (ncov>1) {
      X.all <- cbind(x1.mat[,r], x2.mat[,r], x3.mat[,r], x4.mat[,r], x5.mat[,r])
      y <- 1+xsum+D+D*X.all[,1]*delta+qu
      xv <- cbind(X.all[,1:4],D,D*X.all[,1],X.all[,5])
    }
    if (ncov==0) {
      y <- 1+D*delta+qu
      xv <- cbind(D)
    }

    xmat <- cbind(c(rep(1,n)),xv)
      
    betahat <- rq(y~xv, tau=qtile) 
    betahat.R0 <- rq(y~X.all[,1]+X.all[,2]+X.all[,3]+X.all[,4]+D+D*X.all[,1]+X.all[,5], tau=qtile)
    betahat.R1 <- rq(y~X.all[,1]+X.all[,2]+X.all[,3]+X.all[,4]+D, tau=qtile)
    qtiles <- weg.unif.mat[,r]
    R1 <- rbind(c(rep(0,ncov+1),1,0), c(rep(0,ncov+1),0,1))
    
    # Model 1 Weg (i.e., proposed procedure)
    
    info <- summary.rq(betahat, se="iid", hs=TRUE)$coefficients # Model 1 Wiid
    m <- floor(km*(n/(log(n)^(11/5)))^(5/4))
    hn.list <- eg.proc(R1,xv,y,qtile,a1,a2, m.fixed=FALSE,m, km,qtiles.provided=TRUE,qtiles)
    hn <-matrix(unlist(hn.list[3]),nrow=np,byrow=TRUE)
    jn <- n^(-1)*crossprod(xmat,xmat)
    
    possibleError <- tryCatch({vhat <- qtile*(1-qtile)*ginv(hn) %*% jn %*% ginv(hn)
    }, error=function(e) e)
    if (!inherits(possibleError, "error")){
      rej2.weg[cc-1] <- rej2.weg[cc-1]+(n*t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.weg)
    }

    hn.fixed <- eg.proc.2(R1,xv,y,qtile,a1,a2,km, bw.const, qtiles.provided=TRUE, qtiles) 
    
    possibleError <- tryCatch({vhat <- qtile*(1-qtile)*ginv(hn.fixed) %*% jn %*% ginv(hn.fixed)
    }, error=function(e) e)
    if (!inherits(possibleError, "error")){
      rej2.weg.fixed[cc-1] <- rej2.weg.fixed[cc-1]+(n*t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.weg.fixed)    
    }
    
    
    
    info <- summary.rq(betahat, se="iid", hs=TRUE)$coefficients # Model 1 Wiid
    vhat <- summary.rq(betahat, se="iid", hs=TRUE, covariance=TRUE)$cov
    anova.wiid <- anova(betahat.R0,betahat.R1,test="Wald", se="iid")
    
    rej2.wiid[cc-1] <- rej2.wiid[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.wiid)
    rej2.wiid.anova[cc-1] <- rej2.wiid.anova[cc-1]+(df.test*anova.wiid[[1]][3]>cv951.wiid.anova)

    possibleError <- tryCatch({info <- summary.rq(betahat, se="nid", hs=TRUE)$coefficients # Model 1 Wnid
    }, error=function(e) e)
    if (!inherits(possibleError, "error")){
      vhat <- summary.rq(betahat, se="nid", hs=TRUE, covariance=TRUE)$cov
      anova.wnid <- anova(betahat.R0,betahat.R1,test="Wald", se="nid")
      rej2.wnid[cc-1] <- rej2.wnid[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.wnid)
      rej2.wnid.anova[cc-1] <- rej2.wnid.anova[cc-1]+(df.test*anova.wnid[[1]][3]>cv951.wnid.anova)
    }

    
    info <- summary.rq(betahat, se="ker", hs=TRUE)$coefficients # Model 1 Wker
    vhat <- summary.rq(betahat, se="ker", hs=TRUE, covariance=TRUE)$cov
    anova.wker <- anova(betahat.R0,betahat.R1,test="Wald", se="ker")
    
    rej2.wker[cc-1] <- rej2.wker[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.wker)
    rej2.wker.anova[cc-1] <- rej2.wker.anova[cc-1]+(df.test*anova.wker[[1]][3]>cv951.wker.anova)

    
    
    anova.riid <- anova(betahat.R0,betahat.R1,test="rank")
    tstat1.riid.anova <- sqrt(anova.riid[[1]][3])
    rej2.riid.anova[cc-1] <- rej2.riid.anova[cc-1]+(df.test*anova.riid[[1]][3]>cv951.riid.anova)

    info.boot <- summary.rq(betahat, se="boot", bsmethod="xy", covariance=TRUE, R=boot.reps) # Model 1 Bxy
    info <- info.boot$coefficients 
    vhat <- info.boot$cov
    
    rej2.bxy[cc-1] <- rej2.bxy[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.bxy)

    info.boot <- summary.rq(betahat, se="boot", bsmethod="pwy", covariance=TRUE, R=boot.reps)
    info <- info.boot$coefficients # Model 1 Bpwy
    vhat <- info.boot$cov
    
    rej2.bpwy[cc-1] <- rej2.bpwy[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.bpwy)

    
    possibleError <- tryCatch({info.boot <- summary.rq(betahat, se="boot", bsmethod="mcmb", covariance=TRUE, R=boot.reps)
    }, error=function(e) e)
    if (!inherits(possibleError, "error")){
      info <- info.boot$coefficients # Model 1 Bmcmb
      vhat <- info.boot$cov
      rej2.bmcmb[cc-1] <- rej2.bmcmb[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.bmcmb)
    }
    
    info.boot <- summary.rq(betahat, se="boot", bsmethod="wxy", covariance=TRUE, R=boot.reps)
    info <- info.boot$coefficients # Model 1 Bwxy
    vhat <- info.boot$cov
    
    rej2.bwxy[cc-1] <- rej2.bwxy[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.bwxy)

    info.boot <- summary.rq(betahat, se="boot", bsmethod="wild", covariance=TRUE, R=boot.reps)
    info <- info.boot$coefficients # Model 1 Bwild
    vhat <- info.boot$cov
    
    rej2.bwild[cc-1] <- rej2.bwild[cc-1]+(t(R1 %*% info[,1]) %*% ginv(R1 %*% vhat %*% t(R1)) %*% (R1 %*% info[,1])>cv951.bwild)
  }
}
# End Loop 2

# Compute vectors of empirical powers
rejprob2weg <- as.numeric(rej2.weg/num.reps)
rejprob2weg.fixed <- as.numeric(rej2.weg.fixed/num.reps)
rejprob2wiid <- as.numeric(rej2.wiid/num.reps)
rejprob2wnid <- as.numeric(rej2.wnid/num.reps)
rejprob2wker <- as.numeric(rej2.wker/num.reps)
rejprob2wiid.anova <- as.numeric(rej2.wiid.anova/num.reps)
rejprob2wnid.anova <- as.numeric(rej2.wnid.anova/num.reps)
rejprob2wker.anova <- as.numeric(rej2.wker.anova/num.reps)
rejprob2riid.anova <- as.numeric(rej2.riid.anova/num.reps)
rejprob2bxy <- as.numeric(rej2.bxy/num.reps)
rejprob2bpwy <- as.numeric(rej2.bpwy/num.reps)
rejprob2bmcmb <- as.numeric(rej2.bmcmb/num.reps)
rejprob2bwxy <- as.numeric(rej2.bwxy/num.reps)
rejprob2bwild <- as.numeric(rej2.bwild/num.reps)

rejprob2 <- rbind(rejprob2weg, rejprob2weg.fixed, rejprob2wiid, rejprob2wiid.anova, rejprob2wnid, rejprob2wnid.anova, rejprob2wker, rejprob2wker.anova, rejprob2riid.anova, rejprob2bxy, rejprob2bpwy, rejprob2bmcmb, rejprob2bwxy, rejprob2bwild)
rejprob2 <- round(rejprob2, digits=4)
rownames(rejprob2) <- NULL

rowlabel <- c("weg (fixed)", "wiid", "wnid", "wker", "riid (anova)", "bxy", "bpwy", "bmcmb", "bwxy", "bwild")
size2.raw <- c(size1weg.fixed, size1wiid, size1wnid, size1wker, size1riid.anova, size1bxy, size1bpwy, size1bmcmb, size1bwxy, size1bwild)
size2.raw <- round(size2.raw, digits=4)
rejprob2 <- rbind(rejprob2weg.fixed, rejprob2wiid, rejprob2wnid, rejprob2wker, rejprob2riid.anova, rejprob2bxy, rejprob2bpwy, rejprob2bmcmb, rejprob2bwxy, rejprob2bwild)
rejprob2 <- round(rejprob2, digits=4)
rownames(rejprob2) <- NULL
table <- cbind(rowlabel,size2.raw,rejprob2)

# LaTeX ready table:  The first numeric column gives empirical sizes.  The remaining numeric 
# columns give empirical size-adjusted powers against the alternatives contained in cgrid
xtable(table, caption="QTE, 1000 replications, size and size-adjusted power", digits=c(4))

save(table, file=paste("qdf_qte_int_F2tab2_n",toString(n),"_q",toString(qtile),"_e",toString(err.dist),"_a",toString(alt),".txt",sep=""))

