# This program can be used to reproduce the simulation results appearing in Appendix E.1 
# (supplement p. 27)  of Escanciano & Goh (2018), ``Quantile-Regression Inference with Adaptive Control of Size''

# Users will at minimum need to specify:
#   the sample size ('n'; the paper used n=100 and n=300)
#   the quantile of interest ('qtile'; the paper used qtile=.25, .50, .75)
#   the error distribution ('err.dist'; the paper used err.dist= 1)
#   the specification of the parameter delta; corresponding to the DGPs described in the text as 
#     Model 1, 2, 3, 4, 5 or 6 ('alt'; the paper used alt=1, 2, 3, 4, 5, 6)
#   the leading constant used to specify the number m of random quantiles used; see eq. (14) in the 
#     main text ('km'; the paper used km=1 3, 5, 7, 9, 11, 13, 15)
#   the leading constant in the bandwidth h_m; see eq. (15) in the main text 
#     ('bw.const'; the paper used bw.const=1.00, 1.25, 1.50, 1.75, 2.00)

# Basic parameters
n<- 100           # sample size
qtile <- 0.5      # quantile of interest
err.dist<- 1               # error distribution: 1=normal, 2=t(df1), 3=chi2(df2)
alt<- 1                    # alternatives: 1=loc, 2=loc-scale, 3--5=beta, 6= sin
km <- 5                  # leading constant used in specifying the number $m$ of random quantiles used in our procedure
                         # we have found that km=5 works well in practice
m <- floor(km*(n/(log(n)^(11/5)))^(5/4))  # number of random quantiles used in our procedure
tau0<-0                   # lower bound for alternatives, 0.1 less than qtile
tau1<-1                   # upper bound for alternatives, 0.1 more than qtile
ncov<- 5                  # number of covariates
np <- ncov+2           # number of variables+intercept
cgrid <- c(0,.50,1.0,1.50) # cgrid indexes alternatives
num.reps <- 1000           # number of Monte Carlo replications
df1<-3                    # degrees of freedom for t
df2<-1                    # degrees of freedom for chi2
bw.const <- 1.5           # proportionality constant for fixed bandwidth
                          # we have found that bw.const=1.5 works well in practice

# R libraries that need to be installed on your system
library(quantreg)
library(mvtnorm)
library(MASS)
library(purrr)
library(xtable)

# Endpoints of interval $[a1,a2]$ over which the random quantiles used in our procedure are drawn as 
# iid uniforms, i.e., our procedure uses $m$ iid random variates from a $Unif[a1,a2]$-distribution
a1 <- 0
a2 <- 1

# The eg.proc.2 function implements the proposed estimate of 
# the G matrix for regression quantile standard errors with a fixed bandwidth

eg.proc.2 <- function(R,X,y,qtile,a1,a2,km=1, bw.const=1, qtiles.provided=FALSE, qtiles=NULL) { 
  # X should not contain an intercept
  betahat <- rq(y~X, tau=qtile)
  info <- t(betahat$coefficients)
  n <- length(y)
  Xplus <- cbind(rep(1,n),X)
  d <- dim(Xplus)[2]
  qn <- n^(-1)*crossprod(Xplus,Xplus)
  m <- floor(km*(n/(log(n)^(11/5)))^(5/4))
  
  if (qtiles.provided==FALSE){
    unifs <- runif(m, min=a1,max=a2)
    unifs <- sort(unifs)
  } else {
    unifs <- qtiles
  }
  betahat.grid <- rq(y~X, tau=unifs)
  info.grid <-t(betahat.grid$coefficients)
  
  heg <- abs(bw.const*(log(m)/m)^(1/5))
  
  kern <- .75*(matrix(rep(1,n*m),nrow=n)-(heg^(-1)*(Xplus %*% t(info.grid)-Xplus %*% kronecker(t(info),matrix(rep(1,m),nrow=1))))^2)*(abs(heg^(-1)*(Xplus %*% t(info.grid)-Xplus %*% kronecker(t(info),matrix(rep(1,m),nrow=1))))<=1)
  
  blah <- (a2-a1)*heg^(-1)*kern
  fhateg <- diag(rowMeans(blah))
  ghateg <- n^(-1)*t(Xplus) %*% fhateg %*% Xplus
  return(ghateg)
}

# Initialize vector of test statistics
tstat1.weg.fixed <- numeric(num.reps)




# Initialize total number of rejections across Monte Carlo replications
rej1.weg.fixed <- 0

# Quantities to carry over between Loops 1 and 2
u.mat <- matrix(0,nrow=n,ncol=num.reps)
D.mat <- matrix(0,nrow=n,ncol=num.reps)
x1.mat <- matrix(0,nrow=n,ncol=num.reps)
x2.mat <- matrix(0,nrow=n,ncol=num.reps)
x3.mat <- matrix(0,nrow=n,ncol=num.reps)
x4.mat <- matrix(0,nrow=n,ncol=num.reps)
weg.unif.mat <- matrix(0,nrow=m,ncol=num.reps)


# Loop 1: computation of Wald-type test statistics and their rejections for significance of treatment indicator
for (r in 1:num.reps) {
  cat(paste(" Replication", r, "of", num.reps, "...\n"))
  rand.all <- mvrnorm(n,mu=rep(0,ncov+1),Sigma=diag(rep(1,ncov+1)))
  u <- pnorm(rand.all[,ncov])
  u.mat[,r] <- u
  # Generate quantiles for the error distribution 
  if (err.dist==1) { 
  qu <- qnorm(u)
  }
  if (err.dist==2) { 
  qu <- qt(u,df1)  
  }
  if (err.dist==3) { 
  qu <- qchisq(u, df2)-df2
  }
 # Generate different deltas 
  if (alt<=5) { 
    delta <- 0 
  }
  if (alt==6) {
    delta <- (sin(2*pi*u)-sin(2*pi*qtile))/(2*pi) # sin
  }

  
  DD <- pnorm(rand.all[,ncov+1])
  D <-  qbinom(DD, size=1, prob=0.5)
  D.mat[,r] <- D
  if (ncov>1) {
    X.all <- rand.all[,1:(ncov-1)]
    x1.mat[,r] <- X.all[,1]
    x2.mat[,r] <- X.all[,2]
    x3.mat[,r] <- X.all[,3]
    x4.mat[,r] <- X.all[,4]
    
    xsum=apply(X.all,1,sum)
    y <- 1+xsum+D+D*X.all[,1]*delta+qu
    xv <- cbind(X.all,D,D*X.all[,1])
    xv0 <- cbind(X.all,D)
  }
  
  if (ncov==0) {
    y <- 1+D*delta+qu
    xv <- cbind(D)
  }
     
  xmat <- cbind(c(rep(1,n)),xv)

  betahat <- rq(y~xv, tau=qtile)

  R1 <- rbind(c(rep(0,ncov+1),1))
  qtiles <- runif(m, min=a1,max=a2)
  qtiles <- sort(qtiles)
  weg.unif.mat[,r] <- qtiles
  info <- summary.rq(betahat, se="iid", hs=TRUE)$coefficients # Model 1 Wiid
  
  # Model 1 Weg (i.e., proposed procedure)
    
  jn <- n^(-1)*crossprod(xmat,xmat)

  hn.fixed <- eg.proc.2(R1,xv,y,qtile,a1,a2,km, bw.const,qtiles.provided=TRUE,qtiles) 
  vhat.fixed <- qtile*(1-qtile)*ginv(hn.fixed) %*% jn %*% ginv(hn.fixed)
  se.fixed <- sqrt(diag(n^(-1)*vhat.fixed))
  
  tstat1.weg.fixed[r] <- (info[np,1])/se.fixed[np]
  rej1.weg.fixed <- rej1.weg.fixed +(abs((info[np,1])/se.fixed[np])>1.96)
  

} #end Loop 1

# Empirical size
size1weg.fixed <- rej1.weg.fixed/num.reps

size1.raw <- c(size1weg.fixed)
size1.raw <- round(size1.raw, digits=4)

rowlabel <- c("weg (fixed)")

table.size <- cbind(rowlabel, size1.raw)

# LaTeX ready table:  Empirical sizes for nominal 5\%-tests based on N(0,1)-critical values.  
xtable(table.size, caption="QTE, n=100, 1000 replications, quantile=.5", digits=c(4))


# Empirical tail quantile of Wald-type test statistic.  This is used to compute empirical 
# size-adjusted powers for a .05-level test

cv951.weg.fixed <- quantile(abs(tstat1.weg.fixed),probs=.95)


# Initialize number of rejections under the alternatives considered
rej2.weg.fixed <- seq(0,0,length.out=length(cgrid)-1)


# Loop 2:  Computation of empirical size-adjusted power for nominal 5\% tests of signficance of the 
# treatment indicator.  Power is against each alternative corresponding to the non-zero values in the 
# vector cgrid
for (r in 1:num.reps) {
  cat(paste(" Replication", r, "of", num.reps, "...\n"))
  for (cc in 2:length(cgrid)){
    u <- u.mat[,r]
    
    # Generate quantiles for the error distribution 
    if (err.dist==1) { 
      qu <- qnorm(u)
    }
    else if (err.dist==2) { 
      qu <- qt(u,df1)
    }
    else { 
      qu <- qchisq(u, df2)-df2
    }

    # Generate different alternatives 
    if (alt==1) { 
      delta <- cgrid[c]*(tau0<=u& u<=tau1) # Location
    }
    else if (alt==2) { 
      delta <- cgrid[cc]*(1+qu)*(tau0<=u& u<=tau1) # Location-Scale
    }
    else if (alt==3) { 
      s1<-1
      s2<-4
      qq<-5*qbeta(u,s1,s2)
      delta <- (qbeta(u,s1,s2)-qbeta(qtile,s1,s2)-cgrid[cc]*qq)*(tau0<=u& u<=tau1) # 
    }
    else if (alt==4) { 
      s1<-0.5
      s2<-0.5
      delta <- cgrid[cc]*2*qbeta(u,s1,s2)*(tau0<=u& u<=tau1) # 
    }
    else if (alt==5) { 
      s1<-2
      s2<-2
      delta <- cgrid[cc]*2*qbeta(u,s1,s2)*(tau0<=u& u<=tau1) # 
    }
    else {
      delta <- (sin(2*pi*u)-sin(2*pi*qtile)-cgrid[cc]*2*pi)/(2*pi) # sin
    }
    D <- D.mat[,r]
    if (ncov>1) {
      X.all <- cbind(x1.mat[,r],x2.mat[,r],x3.mat[,r],x4.mat[,r])
      y <- 1+xsum+D+D*X.all[,1]*delta+qu
      xv <- cbind(X.all,D,D*X.all[,1])
    }
    if (ncov==0) {
      y <- 1+D*delta+qu
      xv <- cbind(D)
    }

    xmat <- cbind(c(rep(1,n)),xv)
      
    betahat <- rq(y~xv, tau=qtile) 

    # Model 1 Weg (i.e., proposed procedure)
    
    info <- summary.rq(betahat, se="iid", hs=TRUE)$coefficients # Model 1 Wiid
    R1 <- matrix(c(rep(0,ncov+1),1), nrow=1)
    xmat.minus<-cbind(xv)
    m <- floor(km*(n/(log(n)^(11/5)))^(5/4))
    qtiles <- weg.unif.mat[,r]
    
    jn <- n^(-1)*crossprod(xmat,xmat)

    hn.fixed <- eg.proc.2(R1,xv,y,qtile,a1,a2,km, bw.const,qtiles.provided=TRUE,qtiles) 
    vhat.fixed <- qtile*(1-qtile)*ginv(hn.fixed) %*% jn %*% ginv(hn.fixed)
    se.fixed <- sqrt(diag(n^(-1)*vhat.fixed))
    
    rej2.weg.fixed[cc-1] <- rej2.weg.fixed[cc-1] +(abs((info[np,1])/se.fixed[np])>cv951.weg.fixed)
    
  }
}
# End Loop 2

# Vector of empirical powers
rejprob2weg.fixed <- rej2.weg.fixed/num.reps

rejprob2 <- matrix(rejprob2weg.fixed,nrow=1)
rejprob2 <- round(rejprob2, digits=4)


size2.raw <- matrix(size1weg.fixed,nrow=1)
size2.raw <- round(size2.raw, digits=4)
table <- cbind(size2.raw,rejprob2)
table <- matrix(table,nrow=1)
rownames(table) <- c("weg (fixed)")


# LaTeX ready table:  The first numeric column gives empirical sizes.  The remaining numeric 
# columns give empirical size-adjusted powers against the alternatives contained in cgrid
xtable(table, caption="QTE, 1000 replications, size and size-adjusted power", digits=c(4))

save(table, file=paste("qdf_qte_int_weg_cfixed_tab2_n",toString(n),"_q",toString(qtile),"_e",toString(err.dist),"_a",toString(alt),".txt",sep=""))

